import {ComponentFixture, TestBed, tick} from '@angular/core/testing';

import { ConfirmationComponent } from './confirmation.component';
import {GalleryComponent} from '../gallery/gallery.component';
import {GalleryService} from '../../../services/gallery.service';
import {GuestsService} from '../../../services/guests.service';
import {ConfirmationGroupService} from '../../../services/confirmation-group.service';
import {Message, MessageService} from 'primeng/api';
import {provideRouter} from '@angular/router';
import {provideHttpClient} from '@angular/common/http';
import {provideHttpClientTesting} from '@angular/common/http/testing';
import {GuestType} from '../../../models/guests.type';
import {of, Subject} from 'rxjs';

describe('ConfirmationComponent', () => {

  let component: ConfirmationComponent;
  let fixture: ComponentFixture<ConfirmationComponent>;
  let native: HTMLElement;
  let confirmationGroupService: jasmine.SpyObj<ConfirmationGroupService>;
  let guestService: jasmine.SpyObj<GuestsService>;

  const weddingId = '4d35e8f0-b155-4e79-b5e8-f0b1550e7940';
  const guestId = 'e501bf41-d3d2-417d-81bf-41d3d2217d0a';

  beforeEach(async () => {
    const confirmationGroupServiceSpy = jasmine.createSpyObj<ConfirmationGroupService>('ConfirmationGroupService', ['getConfirmationGroupGuests']);
    const guestServiceSpy = jasmine.createSpyObj<GuestsService>('GuestService', ['patchCharacterPresenceConfirmation']);

    await TestBed.configureTestingModule({
      imports: [
        ConfirmationComponent
      ],
      providers: [
        provideRouter([]),
        provideHttpClient(),
        provideHttpClientTesting(),
        {
          provide: GuestsService,
          useValue: guestServiceSpy
        },
        {
          provide: ConfirmationGroupService,
          useValue: confirmationGroupServiceSpy
        },
        MessageService,
      ]
    })
    .compileComponents();

    fixture = TestBed.createComponent(ConfirmationComponent);
    native = fixture.nativeElement;
    component = fixture.componentInstance;
    component.guestId = guestId;
    component.weddingId = weddingId;
    confirmationGroupService = TestBed.inject(ConfirmationGroupService) as jasmine.SpyObj<ConfirmationGroupService>;
    guestService = TestBed.inject(GuestsService) as jasmine.SpyObj<GuestsService>;

    const guestGroup: GuestType[] = [
      {id: '1', guestAge: '1', guestName: 'name_1', presenceConfirmed: null, weddingConfirmationGroup: {id: '1', name: 'group_1'}},
      {id: '2', guestAge: '1', guestName: 'name_2', presenceConfirmed: true, weddingConfirmationGroup: {id: '1', name: 'group_1'}},
      {id: '3', guestAge: '1', guestName: 'name_3', presenceConfirmed: false, weddingConfirmationGroup: {id: '1', name: 'group_1'}}
    ]

    confirmationGroupServiceSpy.getConfirmationGroupGuests.and.returnValue(of(guestGroup));

    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should render list of guests', () => {
    let containers = native.querySelectorAll('.guest-container')
    expect(containers.length).toBe(3);
  });

  it('should call confirmationButtonAction with correct parameters when confirmation buttons are clicked', () => {
    spyOn(component, 'confirmButtonAction')
    fixture.detectChanges();

    const confirmPresenceButton = native.querySelector('.confirm-presence-button button');
    const confirmAbsenceButton = native.querySelector('.confirm-absence-button button');

    confirmPresenceButton!.dispatchEvent(new Event('click'));
    expect(component.confirmButtonAction).toHaveBeenCalledWith(component.confirmationGuests[0], true);

    confirmAbsenceButton!.dispatchEvent(new Event('click'));
    expect(component.confirmButtonAction).toHaveBeenCalledWith(component.confirmationGuests[0], false);
  });

  it('should call guestService method with correct parameters when confirmation buttons are clicked', () => {
    guestService.patchCharacterPresenceConfirmation.and.returnValue(of(true));
    fixture.detectChanges();

    const confirmPresenceButton = native.querySelector('.confirm-presence-button button');
    const confirmAbsenceButton = native.querySelector('.confirm-absence-button button');

    confirmPresenceButton!.dispatchEvent(new Event('click'));
    expect(guestService.patchCharacterPresenceConfirmation).toHaveBeenCalledWith(weddingId, guestId, true);
    expect(component.confirmationGuests[0].presenceConfirmed).toBe(true);

    confirmAbsenceButton!.dispatchEvent(new Event('click'));
    expect(guestService.patchCharacterPresenceConfirmation).toHaveBeenCalledWith(weddingId, guestId, false);
    expect(component.confirmationGuests[0].presenceConfirmed).toBe(false);
  });

  it('should show confirmation message when confirmation buttons are clicked', () => {
    guestService.patchCharacterPresenceConfirmation.and.returnValue(of(true));
    spyOn(component['messageService'], 'add');
    fixture.detectChanges();

    const confirmPresenceButton = native.querySelector('.confirm-presence-button button');
    const confirmAbsenceButton = native.querySelector('.confirm-absence-button button');

    confirmPresenceButton!.dispatchEvent(new Event('click'));
    expect(component['messageService'].add).toHaveBeenCalledWith(jasmine.objectContaining({
      severity: 'success',
    }));

    confirmAbsenceButton!.dispatchEvent(new Event('click'));
    expect(component['messageService'].add).toHaveBeenCalledWith(jasmine.objectContaining({
      severity: 'warning',
    }));
  });

});
