import {Component, OnInit} from '@angular/core';
import {catchError} from 'rxjs';
import {ConfirmationGroupType, ConfirmationGroupWithGuests} from '../../../models/confirmationGroup.type';
import {GuestsService} from '../../../services/guests.service';
import {GuestType, GuestTypeSimple} from '../../../models/guests.type';
import {Button} from 'primeng/button';
import {TableModule} from 'primeng/table';
import {PaginatorModule} from 'primeng/paginator';
import {CommonModule} from '@angular/common';
import {TagModule} from 'primeng/tag';
import {InputTextModule} from 'primeng/inputtext';
import {InputIconModule} from 'primeng/inputicon';
import {IconFieldModule} from 'primeng/iconfield';
import {CardModule} from 'primeng/card';
import {DividerModule} from 'primeng/divider';
import {ConfirmationService, MessageService} from 'primeng/api';
import {FilterPipe} from '../../../pipes/filter.pipe';
import {ToolbarModule} from 'primeng/toolbar';
import {ConfirmDialogModule} from 'primeng/confirmdialog';
import {MessagesModule} from 'primeng/messages';
import {DialogModule} from 'primeng/dialog';
import {CheckboxModule} from 'primeng/checkbox';
import {RadioButtonModule} from 'primeng/radiobutton';
import {AutoCompleteModule} from 'primeng/autocomplete';
import {MenuModule} from 'primeng/menu';
import {KeycloakService} from '../../../keycloak/keycloak.service';
import {ToastModule} from 'primeng/toast';
import {Ripple} from 'primeng/ripple';
import {SelectButtonModule} from 'primeng/selectbutton';

@Component({
  selector: 'app-guests',
  standalone: true,
  imports: [
    Button,
    TableModule,
    PaginatorModule,
    TagModule,
    InputTextModule,
    IconFieldModule,
    InputIconModule,
    CardModule,
    DividerModule,
    FilterPipe,
    CommonModule,
    ToolbarModule,
    ConfirmDialogModule,
    MessagesModule,
    DialogModule,
    CheckboxModule,
    RadioButtonModule,
    AutoCompleteModule,
    MenuModule,
    ToastModule,
    Ripple,
    SelectButtonModule,
  ],
  providers: [MessageService, ConfirmationService],
  templateUrl: './guests.component.html',
  styleUrl: './guests.component.css',
  host: { ngSkipHydration: 'true' },
})
export class GuestsComponent implements OnInit {

  weddingGroups: ConfirmationGroupWithGuests[] = [];

  searchText:any;

  menuAge: any[] = [
    { name: 'Dorośli', value: 'Dorosły' },
    { name: 'Młodzież', value: 'Młodzież' },
    { name: 'Dzieci', value: 'Dziecko' }
  ];

  filteredGuestsNames: any[] = [];

  groupDialog:boolean = false;
  guestDialog: boolean = false;
  submitted: boolean = false;

  guestForm: GuestType={
    id:'',
    guestName:'',
    presenceConfirmed:null,
    guestAge:'brak',
    weddingConfirmationGroup:{id:'',name:''}
  };

  groupForm: ConfirmationGroupType={
    id:'',
    name:'',
  };

  constructor(private GuestService: GuestsService,
              private confirmationService: ConfirmationService,
              private messageService: MessageService,
              private keycloakService: KeycloakService
  ) {}


  ngOnInit(): void {
    if(this.keycloakService.getToken()){
      this.getGroupsWithGuests();
    }
  }

  getGroupsWithGuests() {
    this.GuestService.getGroupsWithGuestsByWeddingId().pipe(
      catchError((error) => {
        console.error('Error fetching wedding guests:', error);
        throw error;
      })
    ).subscribe((weddingGroupsWithGuests) => {
      this.weddingGroups = weddingGroupsWithGuests;
      console.log(this.weddingGroups);
    });
  }

  //core functions

  addGuest(group:ConfirmationGroupType) {
    this.resetGuestForm()
    this.guestForm.weddingConfirmationGroup.name=group.name;
    this.guestForm.weddingConfirmationGroup.id=group.id;
    this.submitted = false;
    this.guestDialog = true;
  }

  addGroup(){
    this.resetGroupForm();
    this.submitted = false;
    this.groupDialog = true;
  }

  editGuest(guest: GuestType) {
    this.guestForm = { ...guest };
    this.guestDialog = true;
  }

  deleteGuest(guest: GuestType,group:ConfirmationGroupType) {
    this.confirmationService.confirm({
      message: 'Czy na pewno chcesz usunąć tego gościa?',
      header: 'Potwierdzenie',
      acceptLabel: 'Tak, usuń',
      rejectLabel: 'Anuluj',
      accept: () => {
        this.GuestService.deleteGuestInformation(guest).subscribe({
          next: () => {
            this.deleteGuestLocally(group);

            this.messageService.add({
              severity: 'success',
              summary: 'Udało się!',
              detail: 'Gość usunięty',
              life: 3000
            });
          },
          error: () => {
            this.messageService.add({
              severity: 'error',
              summary: 'Błąd!',
              detail: 'Nie udało się usunąć gościa',
              life: 3000
            });
          }
        });
      }
    });
  }

  deleteGuestLocally(group:ConfirmationGroupType) {
    const groupIndex = this.weddingGroups.findIndex(gr =>
      gr.id === group.id
    );

    if (groupIndex !== -1) {
      const guestIndex = this.weddingGroups[groupIndex].guestList.findIndex(
        guest => guest.id === this.guestForm.id
      );

      this.weddingGroups[groupIndex].guestList.splice(guestIndex, 1);
      this.weddingGroups=[... this.weddingGroups];
    }
  }

  deleteGroup(group: ConfirmationGroupType) {
    this.confirmationService.confirm({
      message: 'Czy na pewno chcesz usunąć tą grupę?',
      header: 'Potwierdzenie',
      acceptLabel: 'Tak, usuń',
      rejectLabel: 'Anuluj',
      accept: () => {
        this.GuestService.deleteGroup(group).subscribe({
          next: () => {
            this.deleteGroupLocally(group);

            this.messageService.add({
              severity: 'success',
              summary: 'Udało się!',
              detail: 'Grupa usunięta',
              life: 3000
            });
          },
          error: () => {
            this.messageService.add({
              severity: 'error',
              summary: 'Błąd!',
              detail: 'Nie udało się usunąć grupy',
              life: 3000
            });
          }
        });
      }
    });
  }

  deleteGroupLocally(group:ConfirmationGroupType) {
    const groupIndex = this.weddingGroups.findIndex(gr =>
      gr.id === group.id
    );

    if (groupIndex !== -1) {

      this.weddingGroups.splice(groupIndex, 1);
      this.weddingGroups = [...this.weddingGroups];
    }
  }

  saveGuest() {
    this.submitted = true;

    if (this.guestForm.guestName?.trim()) {
      if (this.guestForm.id) {
        this.saveExistingGuest();
      } else {
        this.saveNewGuest();
      }
    }
  }

  saveExistingGuest() {
    this.GuestService.patchGuestInformation(this.guestForm).subscribe({
      next: () => {
        this.saveExistingGuestLocally();

        this.messageService.add({
          severity: 'success',
          summary: 'Udało się!',
          detail: 'Dane zminenione',
          life: 3000
        });
      },
      error: () => {
        this.messageService.add({
          severity: 'error',
          summary: 'Błąd',
          detail: 'Nie udało się zmienić danych',
          life: 3000
        });
      }
    });
  }

  saveExistingGuestLocally() {
    const groupIndex = this.weddingGroups.findIndex(group =>
      group.guestList.some(guest => guest.id === this.guestForm.id)
    );

    if (groupIndex !== -1) {
      const guestIndex = this.weddingGroups[groupIndex].guestList.findIndex(
        guest => guest.id === this.guestForm.id
      );

      this.weddingGroups[groupIndex].guestList[guestIndex] = { ...this.guestForm };
      this.guestDialog = false;
      this.resetGuestForm();
      this.weddingGroups=[... this.weddingGroups];
    }
  }

  saveNewGuest() {
    this.GuestService.postGuestInformation(this.guestForm).subscribe({
      next: () => {
        this.getGroupsWithGuests();
        this.guestDialog = false;
        this.resetGuestForm();

        this.messageService.add({
          severity: 'success',
          summary: 'Udało się!',
          detail: 'Gość dodany',
          life: 3000
        });
      },
      error: () => {
        this.messageService.add({
          severity: 'error',
          summary: 'Błąd',
          detail: 'Nie udało się utworzyć gościa',
          life: 3000
        });
      }
    });
  }

  saveGroup() {
    this.submitted = true;

    if (this.groupForm.name.trim()) {
        this.GuestService.createGroup(this.groupForm).subscribe({
          next: () => {
            this.getGroupsWithGuests();
            this.groupDialog=false;
            this.groupForm={
              id:'',
              name:'',
            }

            this.messageService.add({
              severity: 'success',
              summary: 'Udało się!',
              detail: 'Grupa utworzona',
              life: 3000
            });
          },
          error: () => {
            this.messageService.add({
              severity: 'error',
              summary: 'Błąd',
              detail: 'Nie udało się utworzyć grupy',
              life: 3000
            });
          }
        });
      }
      this.guestDialog = false;
  }

  hideDialog() {
    this.guestDialog = false;
    this.submitted = false;
    this.groupDialog = false;
  }

  //helping functions

  getPresenceDisplay(isConfirmed: boolean): string {
    return isConfirmed ? 'Tak' : 'Nie';
  }

  getGuestsCountByAccept() {
    return this.weddingGroups
      .flatMap(group=> group.guestList)
      .filter(guest => guest.presenceConfirmed).length;
  }

  getSeverity(status: string) {
    switch (status) {
      case "Dorosły":
        return 'success';
      case "Młodzież":
        return 'info';
      case "Dziecko":
        return 'warning';
      case "Tak":
        return 'success';
      case "Nie":
        return 'danger';
      default:
        return 'info';
    }
  }

  filterGuestsNames(event: any) {
    const query = event.query.toLowerCase();
    this.filteredGuestsNames = Array.from(
      new Set(
        this.weddingGroups
          .flatMap(group => group.guestList)
          .filter((guest) =>
            guest.guestName.toLowerCase().startsWith(query)
          )
          .map((guest: GuestTypeSimple) => guest.guestName)
      )
    );
  }

  filterPresence() {
    this.searchText = this.searchText === 'true' ? '' : 'true';
  }

  resetGuestForm(){
    this.guestForm = {guestAge: 'Brak', presenceConfirmed: null, guestName:'',id:'',weddingConfirmationGroup:{id:'',name:''}};
  }

  resetGroupForm(){
    this.groupForm={id:'',name:''};
  }

}
